/*


Demo fr FATfs und SD-Card

ATmega328 @ 16 MHz (e.g. Arduino Uno) + 1,8'' LCD + SD-Card (includes level shifter)

PB0     LCD_DC
PB1     SD_CS
PB2     LCD_CS
PB3     MOSI
PB4     MISO
PB5     SCK

PC5     LCD_RST
PC4     LCD_BL

*/

#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "main.h"
#include "mmc_avr.h"
#include "uart.h"
#include "ff.h"

volatile uint8_t flag100ms;

FATFS Fatfs;		    /* File system object */
FIL Fil;			    /* File object */
char FatFsdrive[] ="0:";
FRESULT rc;				/* Result code */
DIR dir;				/* Directory object */
FILINFO fno;			/* File information object */

char write_buffer[100] = "This is just a simple text for a file write to SD card. Njoy.";
char mystring[50];

void print_rc (FRESULT rc) {
    char tmp[50];

	uart_puts_P(PSTR("rc="));
    itoa(rc, tmp, 10);
    uart_puts(tmp);
	uart_puts_P(PSTR("\r\n"));
}

int main(void) {
    unsigned int br, bw;

    // IO init
    DDRB  = (1<<PB5) | (1<<PB3) | (1<<PB2) | (1<<PB1) | (1<<PB0);
    PORTB = (1<<PB2) | (1<<PB1);        // CS HIGH, inactive
    DDRC  = (1<<PC5) | (1<<PC4);

	// init UART

	UCSR0B = (1<<TXEN0) | (1<<RXEN0);
    UBRR0H = UBRR_VAL >> 8;
    UBRR0L = UBRR_VAL & 0xFF;

	// timer 1
	// mode 4, CTC, prescaler 8
	TCCR1A  = 0;
	TCCR1B  = (1<<WGM12) | (1<<CS11);
	OCR1A   = PERIOD;
	TIMSK1 |= (1<<OCIE1A);
	
	sei();

	uart_puts_P(PSTR("FATfs demo.\r\n"));

    // Register volume work area, mount now and check FAT existence
    rc = f_mount(&Fatfs, FatFsdrive, 1);

    if (rc != FR_OK) {
        uart_puts_P(PSTR("No SD Card with FAT detected!\r\n"));
        print_rc(rc);
    } else {
        uart_puts_P(PSTR("SD card with FAT detected.\r\n"));
        rc = f_open(&Fil, "/hello.txt", FA_READ);
        if (rc != FR_OK) {
            uart_puts_P(PSTR("File /hello.txt not found!\r\n"));
        } else {
            br = 1;
            while(br != 0) {
                f_read(&Fil, mystring, 49, &br);
                mystring[br]=0;
                uart_puts(mystring);
            }
            f_close(&Fil);            
        }
        
        uart_puts_P(PSTR("Opening file test.txt for write access "));
        rc = f_open(&Fil, "/test.txt", FA_WRITE | FA_CREATE_ALWAYS);
        if (rc != FR_OK) {
            uart_puts_P(PSTR("failed!\r\n"));               
            print_rc(rc);
        } else {
            uart_puts_P(PSTR("OK.\r\n"));
            uart_puts_P(PSTR("Writing file "));
            rc = f_write(&Fil, write_buffer, 100, &bw);        
            if ( rc != FR_OK ) {
                uart_puts_P(PSTR("failed!\r\n"));               
                print_rc(rc);                
            } else {
                uart_puts_P(PSTR("OK.\r\n"));            
            }
        }              
        f_close(&Fil);
    }

    uart_puts_P(PSTR("Demo end.\r\n"));
    
    while(1) {
        if (flag100ms) {
            flag100ms = 0;
            //uart_puts_P(PSTR("."));
        }
    }
    return 0;
}

ISR(TIMER1_COMPA_vect) {

	flag100ms = 1;
    mmc_disk_timerproc();
}
